// Clock parameters
`define TICK      #2
`define PHASE     #5
`define CYCLE     #10

module TOP;
   reg          MRST, CLK;
   reg [3:0]    state;
   // cube
   reg          cube_init;
   wire         cube_busy, cube_done;
   wire [15:0]  cube_Ax, cube_Ay, cube_Az, cube_Bx, cube_By, cube_Bz;

   reg signed [15:0] edge_Ax[0:255], edge_Ay[0:255], edge_Az[0:255],
       edge_Bx[0:255], edge_By[0:255], edge_Bz[0:255];
   reg [7:0]    edge_count;
   reg [7:0]    edge_iterator;
   
   // bresenhem
   reg          bresenhem_init;
   wire         bresenhem_busy, bresenhem_done;
   reg [9:0]    bre_Ax, bre_Ay, bre_Bx, bre_By;

   // Create a 50% duty cycle clock
   initial begin
      CLK = 1;
      forever begin
	       `PHASE CLK = 0;
	       `PHASE CLK = 1;
      end
   end

   bresenhem primary_bresenhem(.CLK(CLK),
                               .reset(MRST),
                               .init(bresenhem_init),
                               .lock(1'b1),
                               .busy(bresenhem_busy),
                               .done(bresenhem_done),
                               .Ax_in(bre_Ax),
                               .Ay_in(bre_Ay),
                               .Bx_in(bre_Bx),
                               .By_in(bre_By));
   
   cube_gen cube_generator(.CLK(CLK), .reset(MRST), .init(cube_init), .lock(1'b1),
                           .busy(cube_busy), .done(cube_done),
                           .Ax(cube_Ax), .Ay(cube_Ay), .Az(cube_Az),
                           .Bx(cube_Bx), .By(cube_By), .Bz(cube_Bz));
   
   // This holds reset long enough to clear all the machine state
   initial begin
      MRST = 1;
      #86 MRST = 0;
      #200
        $finish;
   end

   parameter     SCREEN_TOP = 10'd1,
                 SCREEN_DOWN = 10'd238,
                 SCREEN_LEFT = 10'd3,
                 SCREEN_RIGHT = 10'd319,
                 SCREEN_XHALF = 158,
                 SCREEN_YHALF = 118;
   
   parameter draw_cube = 4'd0,
               wait_cube = 4'd1,
               draw_lines = 4'd2,
               wait_lines = 4'd3;

   // point read from memory. 16 bits wide
   reg signed [15:0] point_Ax, point_Ay, point_Bx, point_By;
   // converted to screen coords
   reg signed [9:0]  screen_Ax, screen_Ay, screen_Bx, screen_By;
   
   always @(posedge CLK) begin
      if (MRST) begin
         $display("Being reset");
         edge_count <= 8'd0;
         edge_iterator <= 8'd0;
         state <= draw_cube;
      end
      else if (state == draw_cube) begin
         cube_init <= 1'b1;

         state <= wait_cube;
      end
      else if (state == wait_cube) begin
         if (cube_busy) begin
            edge_Ax[edge_count] <= cube_Ax;
            edge_Ay[edge_count] <= cube_Ay;
            edge_Az[edge_count] <= cube_Az;
            edge_Bx[edge_count] <= cube_Bx;
            edge_By[edge_count] <= cube_By;
            edge_Bz[edge_count] <= cube_Bz;
            edge_count <= edge_count + 8'd1;

            $display("%d, TICK. init=%b, busy=%b, done=%b, edge_count = %d, [(%h, %h, %h), (%h, %h, %h)] ",
                     $time, cube_init, cube_busy, cube_done, edge_count,
                     cube_Ax, cube_Ay, cube_Az, cube_Bx, cube_By, cube_Bz);
         end // if (cube_busy)
         else if (cube_done) begin
            $display("cube gen is done. advancing to line gen");
            cube_init <= 1'b0;
            state <= draw_lines;
            edge_iterator <= 8'd0;
         end
      end // if (state == wait_cube)
      else if (state == draw_lines) begin
         if (edge_iterator < edge_count) begin
            point_Ax = edge_Ax[edge_iterator];
            screen_Ax = point_Ax[15:6] + SCREEN_XHALF;
            point_Ay = edge_Ay[edge_iterator];
            screen_Ay = point_Ay[15:6] + SCREEN_YHALF;
            point_Bx = edge_Bx[edge_iterator];
            screen_Bx = point_Bx[15:6] + SCREEN_XHALF;
            point_By = edge_By[edge_iterator];
            screen_By = point_By[15:6] + SCREEN_YHALF;

            $display("%d, TICK. init=%b, busy=%b, done=%b, edge_iterator = %d, [(%h, %h, %h), (%h, %h, %h)] [(%d, %d, %d), (%d, %d, %d)] ",
                     $time, cube_init, cube_busy, cube_done, edge_iterator,
                     edge_Ax[edge_iterator], edge_Ay[edge_iterator], edge_Az[edge_iterator],
                     edge_Bx[edge_iterator], edge_By[edge_iterator], edge_Bz[edge_iterator],
                     (edge_Ax[edge_iterator] >>> 10) + SCREEN_XHALF, (edge_Ay[edge_iterator] >>> 10) + SCREEN_YHALF, edge_Az[edge_iterator] >>> 10,
                     (edge_Bx[edge_iterator] >>> 10) + SCREEN_XHALF, (edge_By[edge_iterator] >>> 10) + SCREEN_YHALF, edge_Bz[edge_iterator] >>> 10);

            bresenhem_init <= 1'b1;

            edge_iterator <= edge_iterator + 8'd1;

            //state <= wait_lines;
         end
      end // if (state == draw_lines)
      else if (state == wait_lines) begin
         if (bresenhem_busy) begin
            $display("Waiting for bresenhem");
         end
         else if (bresenhem_done) begin
            $display("bresenhem is done");
            bresenhem_init <= 1'b0;
            state <= draw_lines;
         end
      end
   end // always @ (posedge CLK)

endmodule // TOP
