// Clock parameters
`define TICK      #2
`define PHASE     #5
`define CYCLE     #10

module TOP;
   reg          MRST, CLK;
   reg [3:0]    state;
   // circle
   reg          sphere_init;
   wire         sphere_busy, sphere_done;
   wire [15:0]  sphere_Ax, sphere_Ay, sphere_Az, sphere_Bx, sphere_By, sphere_Bz;
   reg [15:0]   sphere_stepsize, sphere_minorR, sphere_majorR;

   reg signed [15:0] edge_Ax[0:255], edge_Ay[0:255], edge_Az[0:255],
       edge_Bx[0:255], edge_By[0:255], edge_Bz[0:255];
   reg [7:0]    edge_count;
   reg [7:0]    edge_iterator;
   
   // bresenhem
   reg          bresenhem_init;
   wire         bresenhem_busy, bresenhem_done;
   reg [9:0]    bre_Ax, bre_Ay, bre_Bx, bre_By;

   // Create a 50% duty cycle clock
   initial begin
      CLK = 1;
      forever begin
	       `PHASE CLK = 0;
	       `PHASE CLK = 1;
      end
   end

   bresenhem primary_bresenhem(.CLK(CLK),
                               .reset(MRST),
                               .init(bresenhem_init),
                               .lock(1'b1),
                               .busy(bresenhem_busy),
                               .done(bresenhem_done),
                               .Ax_in(bre_Ax),
                               .Ay_in(bre_Ay),
                               .Bx_in(bre_Bx),
                               .By_in(bre_By));
   
   sphere_gen sphere_generator(.CLK(CLK), .reset(MRST), .init(sphere_init), .lock(1'b1),
                               .busy(sphere_busy), .done(sphere_done),
                               .Ax(sphere_Ax), .Ay(sphere_Ay), .Az(sphere_Az),
                               .Bx(sphere_Bx), .By(sphere_By), .Bz(sphere_Bz),
                               .stepsize(sphere_stepsize),
                               .minorR(sphere_minorR),
                               .majorR(sphere_majorR));
   
   // This holds reset long enough to clear all the machine state
   initial begin
      MRST = 1;
      #86 MRST = 0;
      #4200
        $finish;
   end

   parameter     SCREEN_TOP = 10'd1,
                 SCREEN_DOWN = 10'd238,
                 SCREEN_LEFT = 10'd3,
                 SCREEN_RIGHT = 10'd319,
                 SCREEN_XHALF = 158,
                 SCREEN_YHALF = 118;
   
   parameter draw_circle = 4'd0,
               wait_circle = 4'd1,
               draw_lines = 4'd2,
               wait_lines = 4'd3;

   // point read from memory. 16 bits wide
   reg signed [15:0] point_Ax, point_Ay, point_Bx, point_By;
   // converted to screen coords
   reg signed [9:0]  screen_Ax, screen_Ay, screen_Bx, screen_By;
   
   always @(posedge CLK) begin
      if (MRST) begin
         $display("Being reset");
         edge_count <= 8'd0;
         edge_iterator <= 8'd0;
         state <= draw_circle;
      end
      else if (state == draw_circle) begin
         sphere_stepsize <= 16'd64;
         sphere_minorR <= 16'h4000;
         sphere_majorR <= 16'h0000;
         //sphere_stepsize <= 16'd32;
         //sphere_stepsize <= 16'd8;
         sphere_init <= 1'b1;

         state <= wait_circle;
      end
      else if (state == wait_circle) begin
         if (sphere_busy) begin
            edge_Ax[edge_count] <= sphere_Ax;
            edge_Ay[edge_count] <= sphere_Ay;
            edge_Az[edge_count] <= sphere_Az;
            edge_Bx[edge_count] <= sphere_Bx;
            edge_By[edge_count] <= sphere_By;
            edge_Bz[edge_count] <= sphere_Bz;
            edge_count <= edge_count + 8'd1;

            /*
            $display("%d, TICK. init=%b, busy=%b, done=%b, edge_count = %d, [(%h, %h, %h), (%h, %h, %h)] ",
                     $time, sphere_init, sphere_busy, sphere_done, edge_count,
                     sphere_Ax, sphere_Ay, sphere_Az, sphere_Bx, sphere_By, sphere_Bz);
             */
         end // if (sphere_busy)
         else if (sphere_done) begin
            $display("circle gen is done. advancing to line gen");
            sphere_init <= 1'b0;
            state <= draw_lines;
            edge_iterator <= 8'd0;
         end
      end // if (state == wait_circle)
      else if (state == draw_lines) begin
         if (edge_iterator < edge_count) begin
            point_Ax = edge_Ax[edge_iterator];
            screen_Ax = point_Ax[15:6] + SCREEN_XHALF;
            point_Ay = edge_Ay[edge_iterator];
            screen_Ay = point_Ay[15:6] + SCREEN_YHALF;
            point_Bx = edge_Bx[edge_iterator];
            screen_Bx = point_Bx[15:6] + SCREEN_XHALF;
            point_By = edge_By[edge_iterator];
            screen_By = point_By[15:6] + SCREEN_YHALF;

            /*
            $display("%d, TICK. init=%b, busy=%b, done=%b, edge_iterator = %d, [(%h, %h), (%h, %h)] [(%h, %h), (%h, %h)] ",
                     $time, sphere_init, sphere_busy, sphere_done, edge_iterator,
                     screen_Ax, screen_Ay, screen_Bx, screen_By,
                     edge_Ax[edge_iterator], edge_Ay[edge_iterator],
                     edge_Bx[edge_iterator], edge_By[edge_iterator]);
*/
/*
            $display("%d, TICK. init=%b, busy=%b, done=%b, edge_iterator = %d, [(%h, %h, %h), (%h, %h, %h)] [(%d, %d, %d), (%d, %d, %d)] ",
                     $time, sphere_init, sphere_busy, sphere_done, edge_iterator,
                     edge_Ax[edge_iterator], edge_Ay[edge_iterator], edge_Az[edge_iterator],
                     edge_Bx[edge_iterator], edge_By[edge_iterator], edge_Bz[edge_iterator],
                     (edge_Ax[edge_iterator] >>> 10) + SCREEN_XHALF, (edge_Ay[edge_iterator] >>> 10) + SCREEN_YHALF, edge_Az[edge_iterator] >>> 10,
                     (edge_Bx[edge_iterator] >>> 10) + SCREEN_XHALF, (edge_By[edge_iterator] >>> 10) + SCREEN_YHALF, edge_Bz[edge_iterator] >>> 10);
                     //(edge_Ax[edge_iterator] >>> 8) + 10'd17, (edge_Ay[edge_iterator] >>> 8), edge_Az[edge_iterator] >>> 8,
                     //(edge_Bx[edge_iterator] >>> 8) + 10'd17, (edge_By[edge_iterator] >>> 8), edge_Bz[edge_iterator] >>> 8);
 */

            bresenhem_init <= 1'b1;

            edge_iterator <= edge_iterator + 8'd1;

            //state <= wait_lines;
         end
      end // if (state == draw_lines)
      else if (state == wait_lines) begin
         if (bresenhem_busy) begin
            $display("Waiting for bresenhem");
         end
         else if (bresenhem_done) begin
            $display("bresenhem is done");
            bresenhem_init <= 1'b0;
            state <= draw_lines;
         end
      end
   end // always @ (posedge CLK)

endmodule // TOP
